-----------------------------------------------------------------------------
-- FuBar stuff
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Aloft")
local SML = AceLibrary("SharedMedia-1.0")
local roster = AceLibrary:HasInstance("Roster-2.1") and AceLibrary("Roster-2.1") or nil

Aloft = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")

Aloft:RegisterDB("AloftDB")
Aloft:RegisterDefaults('profile', 
	{
		nameplateText =
		{
			visible		= true,
			fontSize	= 10,
			alignment	= "LEFT",
			outline		= "Normal",
			offsetX		= 0,
			offsetY		= 0,
			alpha		= 1,
		},
		healthBar =
		{
			texture			= "Blizzard",
			alpha			= 0.75,
			colorByClass	= true,
			groupOnly		= false,
			colors =
			{
				friendlyPlayer = { 0, 0, 1 },
				friendlyNPC = { 0, 1, 0 },
				friendlyBoss = { 0, 1, 0 },
				hostile = { 1, 0, 0 },
				hostileBoss = { 1, 0, 0 },
				neutral = { 1, 1, 0 },
				partyPet = { 1, 0.5, 1 },
			},
		},
		healthText = 
		{
			visible			= true,
			alpha			= 1,
			point			= "RIGHT",
			relativeToPoint	= "RIGHT",
			offsetX			= 0,
			offsetY			= 0,
			fontSize		= 9,
			outline			= "Normal",
			mode			= "Hidden",
			color			= { 1, 1, 1 }
		},
		overlay =
		{
			visible			= true,
			color			= { 1, 1, 1, 0.25 },
			texture			= "Blizzard"
		},
		highlight =
		{
			color			= { 1, 1, 0, 0.25 },
			texture			= "Blizzard",
			blendMode		= "ADD",
		},
		raidIcon = 
		{
			visible			= true,
			alpha			= 1,
			point			= "RIGHT",
			relativeToPoint	= "LEFT",
			offsetX			= 0,
			offsetY			= 0,
			size			= 12
		},
		bossIcon = 
		{
			visible			= true,
			alpha			= 1.0,
			point			= "CENTER",
			relativeToPoint	= "CENTER",
			offsetX			= 0,
			offsetY			= 0,
			size			= 12
		},
		levelText =
		{
			visible			= true,
			fontSize		= 8,
			outline			= "Normal",
			point			= "LEFT",
			relativeToPoint	= "RIGHT",
			offsetX			= 0,
			offsetY			= 0,
		},
	}
)

Aloft.version = "1.0"
Aloft.hasIcon = "Interface\\Icons\\Spell_Frost_Wisp"	-- Placeholder
Aloft.defaultPosition = 'LEFT'
Aloft.hideWithoutStandby = true
Aloft.independentProfile = true

-----------------------------------------------------------------------------

local numberOfWorldChildren = 0
local nameplateList = { }
local outlineLookup = { ["None"] = nil, ["Normal"] = "OUTLINE", ["Thick"] = "THICKOUTLINE" }
local healthTextMethods =
{
	["Hidden"] = function(healthBar) return "" end,
	["Percent"] = function(healthBar) return string.format("%d%%", floor(100 * healthBar:GetValue() / select(2, healthBar:GetMinMaxValues()))) end,
	["Group Health"] =	function(healthBar) 
							local maxValue = select(2, healthBar:GetMinMaxValues())
							if maxValue == 100 then return "" end
							return healthBar:GetValue()
						end,
	["Group Health Deficit"] =	function(healthBar) 
									local maxValue = select(2, healthBar:GetMinMaxValues())
									if maxValue == 100 then return "" end
									local value = healthBar:GetValue()
									if value == maxValue then return "" end
									return maxValue - value
								end,
}

-----------------------------------------------------------------------------

function Aloft:OnInitialize()
	self:SetText(L["Aloft"])
end

function Aloft:OnEnable()
	self:ScheduleRepeatingEvent("AloftUpdate", self.Update, 0.2, self)
	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateHealthBars()
			self:UpdateHighlights()
			self:UpdateOverlays()
		end
	end)
end

function Aloft:OnDisable()
end

function Aloft:IsNameplateFrame(frame)
	if frame:GetName() then
		return false
	end

	local overlayFrame = select(1, frame:GetRegions())	
	if not overlayFrame or overlayFrame:GetObjectType() ~= "Texture" or overlayFrame:GetTexture() ~= "Interface\\Tooltips\\Nameplate-Border" then
		return false
	end
	
	return true
end

function Aloft:ProcessChildFrames(...)
	for i = 1, select('#', ...) do
		local frame = select(i, ...)
		if self:IsNameplateFrame(frame) then 
			if not nameplateList[frame] then
				self:SetupNamePlate(frame)
				nameplateList[frame] = frame
			end
		end
	end
end

function Aloft:Update()
	local newNumberOfWorldChildren = WorldFrame:GetNumChildren()
	if newNumberOfWorldChildren ~= numberOfWorldChildren then
		numberOfWorldChildren = newNumberOfWorldChildren

		self:ProcessChildFrames(WorldFrame:GetChildren())
	end
end

if Prat_PlayerNames then
	Aloft.PratPlayerNamesUpdateMouseOver = Prat_PlayerNames.updateMouseOver
	Prat_PlayerNames.updateMouseOver = function()
		if UnitIsPlayer("mouseover") and UnitIsFriend("player", "mouseover") and not Prat_PlayerNames.Classes[UnitName("mouseover")] then
			Aloft.PratPlayerNamesUpdateMouseOver(Prat_PlayerNames)
			Aloft:UpdateHealthBarColourForUnitNamed(UnitName("mouseover"))
		end
	end

end

function Aloft:UpdateHealthBarColourForUnitNamed(name)
	for frame in pairs(nameplateList) do	
		if frame.aloftData.name == name then
			self:UpdateNameplate(frame)
		end
	end
end

-----------------------------------------------------------------------------

function Aloft:PlaceFrame(frame, healthBar, placement)
	if placement.visible then
		frame:ClearAllPoints()
		frame:SetPoint(placement.point, healthBar, placement.relativeToPoint, placement.offsetX, placement.offsetY)
		
		if placement.size then
			frame:SetWidth(placement.size)
			frame:SetHeight(placement.size)
		end
		frame:SetAlpha(placement.alpha or 1.0)
	else
		frame:SetAlpha(0)
	end
end

-----------------------------------------------------------------------------

function Aloft:UpdateHealthBarColor(frame, healthBar)
	local overlayFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, bossIconFrame, raidIconFrame = frame:GetRegions()

--	min,max = healthBar:GetMinMaxValues()
--	ChatFrame3:AddMessage("HB: "..frame.aloftData.name.." - "..min.."/"..max)

--	if frame.aloftData.name ~= nameFrame:GetText() then
--		ChatFrame3:AddMessage("Data mismatch in UpdateHealthBarColor: "..tostring(frame.aloftData.name).."/"..tostring(nameFrame:GetText()))
--		ChatFrame3:AddMessage(debugstack())
--	end

	if self.db.profile.healthBar.colorByClass then
		if frame.aloftData.type == "friendlyPlayer" then
			if not self.db.profile.healthBar.groupOnly and Prat_PlayerNames and Prat_PlayerNames.Classes[frame.aloftData.name] then		   
				local color = RAID_CLASS_COLORS[string.upper(Prat_PlayerNames.Classes[frame.aloftData.name])]
				if color then
					healthBar:SetStatusBarColor(color.r, color.g, color.b, self.db.profile.healthBar.alpha)
					return
				end
			end
			if roster then
				local unitid = roster:GetUnitIDFromName(frame.aloftData.name)
				if unitid then
					local color = RAID_CLASS_COLORS[select(2, UnitClass(unitid))]
					if color then
						healthBar:SetStatusBarColor(color.r, color.g, color.b, self.db.profile.healthBar.alpha)
						return
					end
				end
			end
		elseif frame.aloftData.type == "friendlyNPC" then
			if roster and roster:GetUnitIDFromName(frame.aloftData.name) then
				-- Must be a party pet
				local color = self.db.profile.healthBar.colors.partyPet
				healthBar:SetStatusBarColor(color[1], color[2], color[3], self.db.profile.healthBar.alpha)
				return
			end
		end
	end
	
	local color = self.db.profile.healthBar.colors[frame.aloftData.type]
	if color then
		healthBar:SetStatusBarColor(unpack(color))
		healthBar:SetAlpha(self.db.profile.healthBar.alpha)
	else
		healthBar:SetAlpha(self.db.profile.healthBar.alpha)
	end
end

function Aloft:UpdateHealthText(frame, healthBar)
	frame.aloftData.healthTextFrame:SetText(healthTextMethods[self.db.profile.healthText.mode](healthBar))
end

function Aloft:UpdateNameplate(frame)
	local healthBar = frame:GetChildren()

	self:UpdateHealthBarColor(frame, healthBar)
	self:UpdateHealthText(frame, healthBar)
end

function Aloft:OnHealthBarValueChanged(frame)
	local _, _, _, _, nameFrame = frame:GetRegions()
	
	if frame.aloftData.name ~= nameFrame:GetText() then
		self:OnNameplateShow(frame)
	else
		self:UpdateNameplate(frame)
	end
end

function Aloft:OnNameplateShow(frame)
	-- When a nameplate is first shown, grab some data about it
	
	local overlayFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, bossIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar = frame:GetChildren()

	frame.aloftData.name = nameFrame:GetText()
	frame.aloftData.isBoss = bossIconFrame:IsShown()
	frame.aloftData.level = tonumber(levelTextFrame:GetText()) or -1
	if not levelTextFrame:IsShown() then
		frame.aloftData.level = -1
	end

	local r, g, b = healthBar:GetStatusBarColor()
	r, g, b = floor(r * 256), floor(g*256), floor(b*256)

--	ChatFrame3:AddMessage("Showing: "..frame.aloftData.name)
--	ChatFrame3:AddMessage("frame Alpha: "..frame:GetAlpha())
--	ChatFrame3:AddMessage("rgb = "..r.."."..g.."."..b)
	if r == 255 and g == 0 and b == 0 then
		if frame.aloftData.isBoss then
			frame.aloftData.type = "hostileBoss"
		else
			frame.aloftData.type = "hostile"
		end
	elseif r == 0 and g == 0 and b == 255 then
		frame.aloftData.type = "friendlyPlayer"
	elseif r == 0 and g == 255 and b == 0 then
		if frame.aloftData.isBoss then
			frame.aloftData.type = "friendlyBoss"
		else
			frame.aloftData.type = "friendlyNPC"
		end
	elseif r == 255 and g == 255 and b == 0 then
		frame.aloftData.type = "neutral"
	end

--	ChatFrame3:AddMessage(frame.aloftData.name..": "..frame.aloftData.level.. (frame.aloftData.isBoss  and " boss" or " not boss"))

	if not self.db.profile.levelText.visible then
		levelTextFrame:Hide()
	end

-- Testing an idea...
--	nameFrame:SetText("|cffffffff"..frame.aloftData.name)

	self:PlaceFrame(bossIconFrame, healthBar, self.db.profile.bossIcon)
	highlightFrame:SetAllPoints(healthBar)

	-- The frame has an active region for detecting hits that is given roughly by this formula.
	frame:SetHeight(healthBar:GetHeight()*1.6)
	frame:SetWidth(healthBar:GetWidth()*1.05)
	
	self:UpdateNameplate(frame)
end

function Aloft:OnNameplateHide(frame)
	frame.aloftData.name = nil
	frame.aloftData.type = "unknown"
end

-----------------------------------------------------------------------------

local function OnNameplateShow()
	Aloft:OnNameplateShow(this)
end

local function OnNameplateHide()
	Aloft:OnNameplateHide(this)
end

local function OnHealthBarValueChanged()
	Aloft:OnHealthBarValueChanged(this:GetParent())
end

-----------------------------------------------------------------------------

function Aloft:SetupNamePlate(frame)
	frame.aloftData = { type = "unknown" }

	local overlayFrame, frame2, frame3, highlightFrame, nameFrame, levelTextFrame, bossIconFrame, raidIconFrame = frame:GetRegions()
	local healthBar = frame:GetChildren()

	highlightFrame:SetAllPoints(healthBar)
	highlightFrame:SetVertexColor(unpack(self.db.profile.highlight.color))	
	highlightFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.highlight.texture))
	highlightFrame:SetBlendMode(self.db.profile.highlight.blendMode)
	
	nameFrame:SetAllPoints(healthBar)
	-- nameFrame:SetAlpha(self.db.profile.nameplateText.alpha)
	nameFrame:SetFont(nameFrame:GetFont(), self.db.profile.nameplateText.fontSize, outlineLookup[self.db.profile.nameplateText.outline])
	nameFrame:SetJustifyH(self.db.profile.nameplateText.alignment)

	self:PlaceFrame(levelTextFrame, healthBar, self.db.profile.levelText)
	levelTextFrame:SetFont(levelTextFrame:GetFont(), self.db.profile.levelText.fontSize, outlineLookup[self.db.profile.levelText.outline])

	self:PlaceFrame(raidIconFrame, healthBar, self.db.profile.raidIcon)
	self:PlaceFrame(bossIconFrame, healthBar, self.db.profile.bossIcon)

	overlayFrame:SetAllPoints(healthBar)
	overlayFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.overlay.texture))
	overlayFrame:SetVertexColor(unpack(self.db.profile.overlay.color))
	
	if not self.db.profile.overlay.visible then	
		overlayFrame:Hide()
	end
	
	if not self.db.profile.nameplateText.visible then
		nameFrame:Hide()
	end
	
	local healthTextFrame = frame:CreateFontString()

	healthTextFrame:SetFont(nameFrame:GetFont(), self.db.profile.healthText.fontSize, outlineLookup[self.db.profile.healthText.outline])
	healthTextFrame:SetTextColor(unpack(self.db.profile.healthText.color))
	self:PlaceFrame(healthTextFrame, healthBar, self.db.profile.healthText)
	frame.aloftData.healthTextFrame = healthTextFrame

	frame:SetScript("OnShow", OnNameplateShow)
	frame:SetScript("OnHide", OnNameplateHide)

	healthBar:SetScript("OnValueChanged", OnHealthBarValueChanged)
	healthBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.healthBar.texture))

	self:OnNameplateShow(frame)
end

-----------------------------------------------------------------------------

local SML_font = SML:List("font")
local SML_statusbar = SML:List("statusbar")

Aloft.OnMenuRequest = 
{
	handler = Aloft,
	type = 'group',
	args = 
	{
		name =
		{
			type = 'group',
			name = L["Name"],
			desc = L["Nameplate text options"],
			args =
			{
				visible =
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Show unit names"],
					get = function() return Aloft.db.profile.nameplateText.visible end,
					set = function(v) Aloft.db.profile.nameplateText.visible = v Aloft:UpdateNameplateText() end
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the nameplate text"],
					max = 20,
					min = 5,
					step = 1,
					get = function() return Aloft.db.profile.nameplateText.fontSize end,
					set = function(value) Aloft.db.profile.nameplateText.fontSize = value Aloft:UpdateNameplateText() end
				},
				alignment =
				{
					type = 'text',
					name = L["Alignment"],
					desc = L["Sets the alignment of the nameplate text"],
					get = function() return Aloft.db.profile.nameplateText.alignment end,
					set = function(value) Aloft.db.profile.nameplateText.alignment = value Aloft:UpdateNameplateText() end,
					validate = {"LEFT", "CENTER", "RIGHT" },
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the font outline for nameplate text"],
					get = function() return Aloft.db.profile.nameplateText.outline end,
					set = function(value) Aloft.db.profile.nameplateText.outline = value Aloft:UpdateNameplateText() end,
					validate = {"None", "Normal", "Thick" },
				},
				--[[
				alpha =
				{
					type = 'range',
					name = L["Alpha"],
					desc = L["Sets the alpha of the nameplate text"],
					min = 0,
					max = 1.0,
					step = 0.05,
					get = function() return Aloft.db.profile.nameplateText.alpha end,
					set = function(v) Aloft.db.profile.nameplateText.alpha = v Aloft:UpdateNameplateText() end
				},
				]]--
			},
		},

		healthBar =
		{
			type = 'group',
			name = L["Health Bar"],
			desc = L["Health Bar options"],
			args =
			{
				texture =
				{
					type = "text", 
					name = L["Texture"], 
					desc = L["Sets the health bar texture"],
					get = function() return Aloft.db.profile.healthBar.texture end,
					set = function(v) Aloft.db.profile.healthBar.texture = v Aloft:UpdateHealthBars() end,
					validate = SML_statusbar,
				},
				alpha =
				{
					type = 'range',
					name = L["Alpha"],
					desc = L["Sets the health bar alpha"],
					min = 0,
					max = 1.0,
					step = 0.05,
					get = function() return Aloft.db.profile.healthBar.alpha end,
					set = function(v) Aloft.db.profile.healthBar.alpha = v Aloft:UpdateHealthBars() end
				},
				colorByClass =
				{
					type = 'toggle',
					name = L["Color by Class"],
					desc = L["Sets the health bar color according to class. Requires Prat.PlayerNames enabled to show out of group colors"],
					get = function() return Aloft.db.profile.healthBar.colorByClass end,
					set = function(v) Aloft.db.profile.healthBar.colorByClass = v Aloft:UpdateHealthBars() end
				},
				groupOnly =
				{
					type = 'toggle',
					name = L["Group Only"],
					desc = L["Only colors group member's health bars by class"],
					disabled = function() return not Aloft.db.profile.healthBar.colorByClass end,
					get = function() return Aloft.db.profile.healthBar.groupOnly end,
					set = function(v) Aloft.db.profile.healthBar.groupOnly = v Aloft:UpdateHealthBars() end
				},
				colors =
				{
					type = 'group',
					name = L["Colors"],
					desc = L["Customize colors"],
					args =
					{
						friendlyPlayer =
						{
							type = 'color',
							name = L["Friendly Players"],
							desc = L["Sets the health bar color for friendly players"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.friendlyPlayer) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.friendlyPlayer = { r, g, b } Aloft:UpdateHealthBars() end
						},
						friendlyNPC =
						{
							type = 'color',
							name = L["Friendly NPCs"],
							desc = L["Sets the health bar color for friendly NPCs"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.friendlyNPC) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.friendlyNPC = { r, g, b } Aloft:UpdateHealthBars() end
						},
						friendlyBoss =
						{
							type = 'color',
							name = L["Friendly Boss"],
							desc = L["Sets the health bar color for friendly NPC Bosses"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.friendlyBoss) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.friendlyBoss = { r, g, b } Aloft:UpdateHealthBars() end
						},
						hostile =
						{
							type = 'color',
							name = L["Hostile Units"],
							desc = L["Sets the health bar color for hostile units"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.hostile) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.hostile = { r, g, b } Aloft:UpdateHealthBars() end
						},
						hostileBoss =
						{
							type = 'color',
							name = L["Hostile Boss"],
							desc = L["Sets the health bar color for hostile boss units"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.hostileBoss) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.hostileBoss = { r, g, b } Aloft:UpdateHealthBars() end
						},
						neutral =
						{
							type = 'color',
							name = L["Neutral Units"],
							desc = L["Sets the health bar color for neutral units"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.neutral) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.neutral = { r, g, b } Aloft:UpdateHealthBars() end
						},
						partyPet =
						{
							type = 'color',
							name = L["Party Pets"],
							desc = L["Sets the health bar color for party pets"],
							get = function() return unpack(Aloft.db.profile.healthBar.colors.partyPet) end,
							set = function(r, g, b) Aloft.db.profile.healthBar.colors.partyPet = { r, g, b } Aloft:UpdateHealthBars() end
						},
					}
				}
			},
		},

		healthText =
		{
			type = 'group',
			name = L["Health Text"],
			desc = L["Health text options"],
			args =
			{
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the health text"],
					max = 20,
					min = 5,
					step = 1,
					get = function() return Aloft.db.profile.healthText.fontSize end,
					set = function(value) Aloft.db.profile.healthText.fontSize = value Aloft:UpdateHealthTexts() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the outline for health text"],
					get = function() return Aloft.db.profile.healthText.outline end,
					set = function(value) Aloft.db.profile.healthText.outline = value Aloft:UpdateHealthTexts() end,
					validate = {"None", "Normal", "Thick"},
				},
				mode =
				{
					type = 'text',
					name = L["Mode"],
					desc = L["Choose what to show for health text"],
					get = function() return Aloft.db.profile.healthText.mode end,
					set = function(value) Aloft.db.profile.healthText.mode = value Aloft:UpdateHealthTexts() end,
					validate = {"Hidden", "Percent", "Group Health", "Group Health Deficit"},
				},
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the health text"],
					get = function() return Aloft.db.profile.healthText.point end,
					set = function(v) Aloft.db.profile.healthText.point = v Aloft:UpdateHealthTexts() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the health text"],
					get = function() return Aloft.db.profile.healthText.relativeToPoint end,
					set = function(v) Aloft.db.profile.healthText.relativeToPoint = v Aloft:UpdateHealthTexts() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the health text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.healthText.offsetX end,
					set = function(v) Aloft.db.profile.healthText.offsetX = v Aloft:UpdateHealthTexts() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the health text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.healthText.offsetY end,
					set = function(v) Aloft.db.profile.healthText.offsetY = v Aloft:UpdateHealthTexts() end
				},
				alpha =
				{
					type = 'range',
					name = L["Alpha"],
					desc = L["Sets the alpha of the health text"],
					min = 0,
					max = 1.0,
					step = 0.05,
					get = function() return Aloft.db.profile.healthText.alpha end,
					set = function(v) Aloft.db.profile.healthText.alpha = v Aloft:UpdateHealthTexts() end
				},
				color =
				{
					type = 'color',
					name = L["Color"],
					desc = L["Sets the health text color"],
					get = function() return unpack(Aloft.db.profile.healthText.color) end,
					set = function(r, g, b, a) Aloft.db.profile.healthText.color = { r, g, b, a } Aloft:UpdateHealthTexts() end,
				},
			},
		},

		levelText =
		{
			type = 'group',
			name = L["Level Text"],
			desc = L["Level text options"],
			args =
			{
				visible = 
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable level text display on the nameplate"],
					get = function() return Aloft.db.profile.levelText.visible end,
					set = function(v) Aloft.db.profile.levelText.visible = v Aloft:UpdateLevelText() end
				},
				fontSize =
				{
					type = 'range',
					name = L["Font Size"],
					desc = L["Sets the font height of the level text"],
					max = 20,
					min = 5,
					step = 1,
					get = function() return Aloft.db.profile.levelText.fontSize end,
					set = function(value) Aloft.db.profile.levelText.fontSize = value Aloft:UpdateLevelText() end
				},
				outline =
				{
					type = 'text',
					name = L["Outline"],
					desc = L["Sets the font height of the level text"],
					get = function() return Aloft.db.profile.levelText.outline end,
					set = function(value) Aloft.db.profile.levelText.outline = value Aloft:UpdateLevelText() end,
					validate = {"None", "Normal", "Thick" },
				},
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the level text"],
					get = function() return Aloft.db.profile.levelText.point end,
					set = function(v) Aloft.db.profile.levelText.point = v Aloft:UpdateLevelText() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the level text"],
					get = function() return Aloft.db.profile.levelText.relativeToPoint end,
					set = function(v) Aloft.db.profile.levelText.relativeToPoint = v Aloft:UpdateLevelText() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the level text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.levelText.offsetX end,
					set = function(v) Aloft.db.profile.levelText.offsetX = v Aloft:UpdateLevelText() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the level text"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.levelText.offsetY end,
					set = function(v) Aloft.db.profile.levelText.offsetY = v Aloft:UpdateLevelText() end
				},
			},
		},

		overlay =
		{
			type = 'group',
			name = L["Overlay"],
			desc = L["Overlay options"],
			args =
			{
				visible =
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Show an overlay on the full health bar"],
					get = function() return Aloft.db.profile.overlay.visible end,
					set = function(v) Aloft.db.profile.overlay.visible = v Aloft:UpdateOverlays() end
				},
				color =
				{
					type = 'color',
					name = L["Color"],
					desc = L["Sets the overlay color"],
					disabled  = function() return not Aloft.db.profile.overlay.visible end,
					get = function() return unpack(Aloft.db.profile.overlay.color) end,
					set = function(r, g, b, a) Aloft.db.profile.overlay.color = { r, g, b, a } Aloft:UpdateOverlays() end,
					hasAlpha = true
				},
				texture =
				{
					type = "text", 
					name = L["Texture"], 
					desc = L["Sets the overlay texture"],
					get = function() return Aloft.db.profile.overlay.texture end,
					set = function(v) Aloft.db.profile.overlay.texture = v Aloft:UpdateOverlays() end,
					validate = SML_statusbar,
				},
			},
		},
	
		highlight =
		{
			type = 'group',
			name = L["Highlight"],
			desc = L["Mouseover highlight options"],
			args =
			{
				color =
				{
					type = 'color',
					name = L["Color"],
					desc = L["Sets the highlight color"],
					get = function() return unpack(Aloft.db.profile.highlight.color) end,
					set = function(r, g, b, a) Aloft.db.profile.highlight.color = { r, g, b, a } Aloft:UpdateHighlights() end,
					hasAlpha = true
				},
				texture =
				{
					type = "text", 
					name = L["Texture"], 
					desc = L["Sets the highlight texture"],
					get = function() return Aloft.db.profile.highlight.texture end,
					set = function(v) Aloft.db.profile.highlight.texture = v Aloft:UpdateHighlights() end,
					validate = SML_statusbar,
				},
				mode =
				{
					type = "text",
					name = L["Blend Mode"],
					desc = L["Sets the blend mode for the highlight texture"],
					get = function() return Aloft.db.profile.highlight.blendMode end,
					set = function(v) Aloft.db.profile.highlight.blendMode = v Aloft:UpdateHighlights() end,
					validate = { "BLEND", "ADD" },
				},
			},
		},
	
		raidIcon =
		{
			type = 'group',
			name = L["Raid Icon"],
			desc = L["Raid icon placement options"],
			args =
			{
				visible = 
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable raid icon display on the nameplate"],
					get = function() return Aloft.db.profile.raidIcon.visible end,
					set = function(v) Aloft.db.profile.raidIcon.visible = v Aloft:UpdateRaidIcons() end
				},
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the raid icon"],
					get = function() return Aloft.db.profile.raidIcon.point end,
					set = function(v) Aloft.db.profile.raidIcon.point = v Aloft:UpdateRaidIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the raid icon"],
					get = function() return Aloft.db.profile.raidIcon.relativeToPoint end,
					set = function(v) Aloft.db.profile.raidIcon.relativeToPoint = v Aloft:UpdateRaidIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				size =
				{
					type = 'range',
					name = L["Size"],
					desc = L["Size in pixels of the raid icon"],
					min = 4,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.raidIcon.size end,
					set = function(v) Aloft.db.profile.raidIcon.size = v Aloft:UpdateRaidIcons() end
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the raid icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.raidIcon.offsetX end,
					set = function(v) Aloft.db.profile.raidIcon.offsetX = v Aloft:UpdateRaidIcons() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the raid icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.raidIcon.offsetY end,
					set = function(v) Aloft.db.profile.raidIcon.offsetY = v Aloft:UpdateRaidIcons() end
				},
				alpha =
				{
					type = 'range',
					name = L["Alpha"],
					desc = L["Sets the raid icon alpha"],
					min = 0,
					max = 1.0,
					step = 0.05,
					get = function() return Aloft.db.profile.raidIcon.alpha end,
					set = function(v) Aloft.db.profile.raidIcon.alpha = v Aloft:UpdateRaidIcons() end
				},
			},
		},

		bossIcon =
		{
			type = 'group',
			name = L["Boss Icon"],
			desc = L["Boss icon placement options"],
			args =
			{
				visible = 
				{
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable boss icon display on the nameplate"],
					get = function() return Aloft.db.profile.bossIcon.visible end,
					set = function(v) Aloft.db.profile.bossIcon.visible = v Aloft:UpdateBossIcons() end
				},
				anchor = {
					type = "text", 
					name = L["Anchor"], 
					desc = L["Sets the anchor for the boss icon"],
					get = function() return Aloft.db.profile.bossIcon.point end,
					set = function(v) Aloft.db.profile.bossIcon.point = v Aloft:UpdateBossIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				anchorto = {
					type = "text", 
					name = L["Anchor To"], 
					desc = L["Sets the relative point on the health bar to anchor the boss icon"],
					get = function() return Aloft.db.profile.bossIcon.relativeToPoint end,
					set = function(v) Aloft.db.profile.bossIcon.relativeToPoint = v Aloft:UpdateBossIcons() end,
					validate = {"TOPLEFT", "TOP", "TOPRIGHT", "LEFT", "CENTER", "RIGHT", "BOTTOMLEFT", "BOTTOM", "BOTTOMRIGHT"},
				},
				size =
				{
					type = 'range',
					name = L["Size"],
					desc = L["Size in pixels of the boss icon"],
					min = 4,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.bossIcon.size end,
					set = function(v) Aloft.db.profile.bossIcon.size = v Aloft:UpdateBossIcons() end
				},
				offsetX =
				{
					type = 'range',
					name = L["X Offset"],
					desc = L["X offset of the boss icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.bossIcon.offsetX end,
					set = function(v) Aloft.db.profile.bossIcon.offsetX = v Aloft:UpdateBossIcons() end
				},
				offsetY =
				{
					type = 'range',
					name = L["Y Offset"],
					desc = L["Y offset of the boss icon"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return Aloft.db.profile.bossIcon.offsetY end,
					set = function(v) Aloft.db.profile.bossIcon.offsetY = v Aloft:UpdateBossIcons() end
				},
				alpha =
				{
					type = 'range',
					name = L["Alpha"],
					desc = L["Sets the boss icon alpha"],
					min = 0,
					max = 1.0,
					step = 0.05,
					get = function() return Aloft.db.profile.bossIcon.alpha end,
					set = function(v) Aloft.db.profile.bossIcon.alpha = v Aloft:UpdateBossIcons() end
				},
			},
		},
	},
}

Aloft:RegisterChatCommand({"/aloft"}, Aloft.OnMenuRequest)

-----------------------------------------------------------------------------

function Aloft:UpdateNameplateText()
	for frame in pairs(nameplateList) do	
		local _, _, _, _, nameFrame = frame:GetRegions()
	
		if self.db.profile.nameplateText.visible then
			nameFrame:Show()
		else
			nameFrame:Hide()
		end
	
		-- nameFrame:SetAlpha(self.db.profile.nameplateText.alpha)
		nameFrame:SetFont(nameFrame:GetFont(), self.db.profile.nameplateText.fontSize, outlineLookup[self.db.profile.nameplateText.outline])
		nameFrame:SetJustifyH(self.db.profile.nameplateText.alignment)
	end
end

function Aloft:UpdateHealthBars()
	for frame in pairs(nameplateList) do	
		local overlayFrame = frame:GetRegions()
		local healthBar = frame:GetChildren()
		healthBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.healthBar.texture))
		healthBar:SetAlpha(self.db.profile.healthBar.alpha)
		if frame.aloftData.name then
			self:UpdateHealthBarColor(frame, healthBar)
		end
	end
end

function Aloft:UpdateHealthTexts()
	for frame in pairs(nameplateList) do
		local healthBar = frame:GetChildren()
		
		local healthTextFrame = frame.aloftData.healthTextFrame
		healthTextFrame:SetFont(healthTextFrame:GetFont(), self.db.profile.healthText.fontSize, outlineLookup[self.db.profile.healthText.outline])
		healthTextFrame:SetTextColor(unpack(self.db.profile.healthText.color))
		self:PlaceFrame(healthTextFrame, healthBar, self.db.profile.healthText)
		self:UpdateHealthText(frame, healthBar)
	end
end

function Aloft:UpdateLevelText()
	for frame in pairs(nameplateList) do
		local _, _, _, _, _, levelTextFrame = frame:GetRegions()
		local healthBar = frame:GetChildren()
		
		if self.db.profile.levelText.visible then
			levelTextFrame:Show()
		else
			levelTextFrame:Hide()
		end
		levelTextFrame:SetFont(levelTextFrame:GetFont(), self.db.profile.levelText.fontSize, outlineLookup[self.db.profile.levelText.outline])
		self:PlaceFrame(levelTextFrame, healthBar, self.db.profile.levelText)
	end
end

function Aloft:UpdateRaidIcons()
	for frame in pairs(nameplateList) do	
		local _, _, _, _, _, _, _, raidIconFrame = frame:GetRegions()
		local healthBar = frame:GetChildren()
	
		Aloft:PlaceFrame(raidIconFrame, healthBar, self.db.profile.raidIcon)
	end
end

function Aloft:UpdateBossIcons()
	for frame in pairs(nameplateList) do	
		local _, _, _, _, _, _, bossIconFrame = frame:GetRegions()
		local healthBar = frame:GetChildren()

		Aloft:PlaceFrame(bossIconFrame, healthBar, self.db.profile.bossIcon)
	end
end

function Aloft:UpdateOverlays()
	for frame in pairs(nameplateList) do
		local overlayFrame = frame:GetRegions()
		if self.db.profile.overlay.visible then
			overlayFrame:Show()
		else
			overlayFrame:Hide()
		end
		overlayFrame:SetVertexColor(unpack(self.db.profile.overlay.color))
		overlayFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.overlay.texture))
	end
end

function Aloft:UpdateHighlights()
	for frame in pairs(nameplateList) do
		local _, _, _, highlightFrame = frame:GetRegions()
		highlightFrame:SetVertexColor(unpack(self.db.profile.highlight.color))
		highlightFrame:SetTexture(SML:Fetch("statusbar", self.db.profile.highlight.texture))
		highlightFrame:SetBlendMode(self.db.profile.highlight.blendMode)
	end
end

-----------------------------------------------------------------------------
